#!/usr/bin/env python
import sys, os
import getopt
import commands
import re

class Options:
    filename = ''
    tcp = 0
    udp = 0
opts = Options

gt_table = {}
gt_table_group = {}
gt_table_named_destination = {}
gt_table_port_usage = {}

tot_sess = {"unknown":{}, "equal":{}, "different":{}, "nomatch":{}, "ambiguous":{}, "ambiguoussquare":{} }
tot_cat = {"times": {"unknown":0, "equal":0, "different":0, "nomatch":0, "ambiguous":0, "ambiguoussquare":0 },
           "bytes": {"unknown":0, "equal":0, "different":0, "nomatch":0, "ambiguous":0, "ambiguoussquare":0 }}
tot = {"times":0, "bytes":0}

noname_table = {}

table3_sess = [ [0, 0, 0], [0, 0, 0], [0, 0, 0] ]
table3_byte = [ [0, 0, 0], [0, 0, 0], [0, 0, 0] ]

def doprocess():

    table3_1_1_eq_sess = 0
    table3_1_1_eq_byte = 0
    noname_notresolved = 0
    noname_resolved_by_service = 0
    
    # open input file
    nomefile = os.path.expanduser(opts.filename)

    try:
        fid = open(nomefile,mode='r')
    except:
        sys.stderr.write("Cannot open file for reading")
        sys.exit(-1)

    lines = []
    
    # read all the file and store in memory
    for line in fid.readlines():

        # extract a new line and split its fields
        fields = line.strip('\n').split(":")
        (ts, ipsrc, ipdst, portsrc, portdst, l7_verdict, gt_verdict, l7_subverdict, t1, t2, bs, bc, l4proto) = fields
        ts = int(ts)

        # collect some applications
        
        # SKYPE
        pat = re.compile("skype", re.IGNORECASE)
        if pat.findall(gt_verdict):
            gt_verdict = "SKYPE"

        # WEBBROWSER
        pat = re.compile("firefox|iexplore|safari", re.IGNORECASE)
        if pat.findall(gt_verdict):
            gt_verdict = "WEBBROWSER"

        # MAIL
        pat = re.compile("thunderbird|mail|eudora|evolution", re.IGNORECASE)
        if pat.findall(gt_verdict):
            gt_verdict = "MAILCLIENT"
        
        if opts.tcp and not (l4proto == "TCP"):
            continue
        if opts.udp and not (l4proto == "UDP"):
            continue
 
        # skip session whose duration is null
        if float(t2) == 0.0:
            continue

        # skip TCP sessions that have at least one direction without packets
        if l4proto == "TCP" and (int(bs) == 0 or int(bc) == 0):
            continue

        # -----------------------------------------------------------------------------------------
        # PART 1: sessions without a gt tag, skip them
        if gt_verdict == "NoName" and float(t2) < 1:
            continue
        if gt_verdict == "NoName":
            continue

        tot["times"] += 1
        tot["bytes"] += int(bc) + int(bs)
        
        # -----------------------------------------------------------------------------------------
        # PART 2: populate matrix with statistics
        
        l7_verdict_num = len(l7_verdict.split(';')) - 1
        l7_subverdict_num = len(l7_subverdict.split(';')) - 1

        if l7_verdict_num > 2:
            l7_verdict_num = 2

        if l7_subverdict_num > 2:
            l7_subverdict_num = 2
            
        cat = None
        catbis = None

        # TEST 1, tabella nuova
        
        if l7_verdict == "NOMATCH":
            catbis = (0, 0)

        else:
            if l7_subverdict == "proto_not_found" or l7_subverdict == "NOMATCH":
                catbis = (l7_verdict_num, 0)
            else:
                catbis = (l7_verdict_num, l7_subverdict_num)

        (n, m) = catbis

        if(n == 1 and n == m and l7_verdict == l7_subverdict):
            table3_1_1_eq_sess += 1
            table3_1_1_eq_byte += int(bs) + int(bc)
            
        table3_sess[n][m] += 1
        table3_byte[n][m] += int(bs) + int(bc)

        # verify we do not have any app_not_found
        dietest = re.compile("app_not_found", re.IGNORECASE)
        if dietest.findall(l7_subverdict):
            sys.stdout.write("IMPOSSIBILE, check!")
            sys.exit(-1)
            
        # fill in per category table
    
        if l7_verdict == "NOMATCH":
            cat = "unknown"
            
        elif l7_subverdict == "proto_not_found;":
            cat = "nomatch"
            
        elif l7_subverdict == "NOMATCH":
            cat = "nomatch"
            
        elif l7_verdict == l7_subverdict and l7_verdict_num == 1:
            cat = "equal"

        elif l7_verdict != l7_subverdict and l7_verdict_num == 1 and l7_subverdict_num == 1:
            cat = "different"

        elif l7_verdict_num > 1 and l7_subverdict_num == 1:
            cat = "ambiguous"

        elif l7_verdict_num > 1 and l7_subverdict_num > 1:
            cat = "ambiguoussquare"

        tot_cat["times"][cat] += 1
        tot_cat["bytes"][cat] += int(bs) + int(bc)
        
        # -----------------------------------------------------------------------------------------
        # PART 3: track totals
        if not tot_sess[cat].has_key(gt_verdict):
            tot_sess[cat][gt_verdict] = {}
            tot_sess[cat][gt_verdict]["times"] = 0
            tot_sess[cat][gt_verdict]["bytes"] = 0

        tot_sess[cat][gt_verdict]["times"] += 1
        tot_sess[cat][gt_verdict]["bytes"] += int(bc) + int(bs)

        # -----------------------------------------------------------------------------------------
        # PART 4: statistics per application
        if not gt_table.has_key(gt_verdict):
            gt_table[gt_verdict] = {}
            gt_table[gt_verdict]["times"] = 0
            gt_table[gt_verdict]["bytes"] = 0
            gt_table[gt_verdict]["sess"] = { "unknown":0, "equal":0, "different":0, "nomatch":0, "ambiguous":0, "ambiguoussquare":0 }
            gt_table[gt_verdict]["byte"] = { "unknown":0, "equal":0, "different":0, "nomatch":0, "ambiguous":0, "ambiguoussquare":0 }
                
        gt_table[gt_verdict]["times"] += 1
        gt_table[gt_verdict]["bytes"] += int(bc) + int(bs)

        gt_table[gt_verdict]["sess"][cat] += 1
        gt_table[gt_verdict]["byte"][cat] += int(bc) + int(bs)
    
    # close the file
    fid.close()

    # PART1 ----------------------------------------------------------------------------
    print "PART1-----------------------------------------------------------------------------"
    sys.stdout.write("sessions %d\n" % tot["times"])
    sys.stdout.write("bytes %d\n" % tot["bytes"])

    # PART2 ----------------------------------------------------------------------------
    print "PART2-----------------------------------------------------------------------------"

    catsdisplay = sorted(tot_cat["times"].items(), lambda x, y: cmp(x[1], y[1]), reverse = True)
    sys.stdout.write("sessions:\n")
    for cat in catsdisplay:
        catname = cat[0]
        sessionsno = cat[1]
        sys.stdout.write('%s %d\n' % (catname, sessionsno))

    # PART3 ----------------------------------------------------------------------------
    print "PART3-----------------------------------------------------------------------------"

    catsdisplay = sorted(tot_cat["bytes"].items(), lambda x, y: cmp(x[1], y[1]), reverse = True)
    sys.stdout.write("bytes:\n")
    for cat in catsdisplay:
        catname = cat[0]
        bytesno = cat[1]
        sys.stdout.write('%s %d\n' % (catname, bytesno))


    # PART4 ----------------------------------------------------------------------------
    print "PART4-----------------------------------------------------------------------------"

    for cat in tot_sess.keys():

        print "%s detail sessions\n------------------------" % cat
        
        tot_sess_cat = tot_sess[cat]
        appsdisplay = sorted(tot_sess_cat.items(), lambda x, y: cmp(x[1]["times"], y[1]["times"]), reverse = True)

        for app in appsdisplay:
            appname = app[0]
            sessionsno = app[1]["times"]
            perc = float(sessionsno)/float(tot["times"])*100
            
            sys.stdout.write("%s = %d - %f 0/0\n" % (appname, sessionsno, perc))

        print " "

    # PART5 ----------------------------------------------------------------------------
    print "PART5-----------------------------------------------------------------------------"

    for cat in tot_sess.keys():

        print "%s detail bytes\n------------------------" % cat
        
        tot_sess_cat = tot_sess[cat]
        appsdisplay = sorted(tot_sess_cat.items(), lambda x, y: cmp(x[1]["bytes"], y[1]["bytes"]), reverse = True)

        for app in appsdisplay:
            appname = app[0]
            bytesno = app[1]["bytes"]
            perc = float(bytesno)/float(tot["bytes"])*100
            
            sys.stdout.write("%s = %d - %f 0/0\n" % (appname, bytesno, perc))

        print " "

    # PART6 ----------------------------------------------------------------------------
    # emit applications statistics
    print "PART6-----------------------------------------------------------------------------"

    for app in gt_table.keys():

        totsessapp = 0
        for cat in gt_table[app]["sess"]:
            totsessapp += gt_table[app]["sess"][cat]

        if totsessapp == gt_table[app]["sess"]["equal"]:
            continue

        sys.stdout.write("-----------------------------\n ** %s\n\n" % app)

        sys.stdout.write("sessions: %d\n" % totsessapp)
        for cat in gt_table[app]["sess"]:
            perc_on_totsessapp = float(gt_table[app]["sess"][cat]) / float(totsessapp) * 100.0

            sys.stdout.write("%s: %f\n" % (cat, perc_on_totsessapp))

        sys.stdout.write("\n")
        
        totbyteapp = 0
        for cat in gt_table[app]["byte"]:
            totbyteapp += gt_table[app]["byte"][cat]

        sys.stdout.write("bytes: %d\n" % totbyteapp)
        for cat in gt_table[app]["byte"]:
            perc_on_totbyteapp = float(gt_table[app]["byte"][cat]) / float(totbyteapp) * 100.0

            sys.stdout.write("%s: %f\n" % (cat, perc_on_totbyteapp))

        sys.stdout.write("\n")

    # PART7 ----------------------------------------------------------------------------
    # emit table statistics
    print "PARTE7-----------------------------------------------------------------------------"

    sys.stdout.write("\nsessions:\n")
    for n in [0, 1, 2]:
        for m in [0, 1, 2]:
            sys.stdout.write("%0.2f\t" % (float(table3_sess[n][m]) / float(tot["times"]) * 100.0 ) )
        sys.stdout.write("\n")

    sys.stdout.write("n=m=1 and verdicts equal happens %0.2f times\n" % (float(table3_1_1_eq_sess) / float(tot["times"]) * 100.0 ) )

    sys.stdout.write("\nbytes:\n")
    for n in [0, 1, 2]:
        for m in [0, 1, 2]:
            sys.stdout.write("%0.2f\t" % (float(table3_byte[n][m]) / float(tot["bytes"]) * 100.0 ) )
        sys.stdout.write("\n")

    sys.stdout.write("n=m=1 and verdicts equal happens %0.2f times\n" % (float(table3_1_1_eq_byte) / float(tot["bytes"]) * 100.0 ))

if __name__=="__main__":
    try:
        options, arguments = getopt.getopt(sys.argv[1:], "r:p:c:tu")
        
    except getopt.GetoptError:
        sys.exit(2)
    
    opts.part = None

    for o, a in options:
        if o in ("-r",):
            opts.filename = a
        if o in ("-t",):
            opts.tcp = 1
        if o in ("-u",):
            opts.udp = 1
            
    doprocess()
