##!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#  ABOUT

"""
 *
 *           f l o w . p y
 *
 * flow.py contains the code to handle TCP flow object
 *
 * Author -
 *
 *      Francesco Gringoli
 *      francesco.gringoli@ing.unibs.it
 *      University of Brescia
 *      1/3/2008
 *
 * Status -
 *      Stable
 * 
"""

#-------------------------------------------------------------------------------
# This file handles flows

#-------------------------------------------------------------------------------
# TODO
#
# - move 'idx' handling somewhere else...
# - improve handling of the log list the passed object (what is?)
# - add comments to class Flow's methods
# - why we need time

#-------------------------------------------------------------------------------
# import section

import pcap, dpkt, dnet
import time
import sys

#-------------------------------------------------------------------------------
# This class defines the data type 'Flow'
#
# Method 'setcurl2pkt' ...
# Method 'blabla' ...
#
# STATUS add comments

flow_modules = []

class flow:
    idx=0

    def __init__( self, ippkt_ = None ):

        if ippkt_ == None:
            self._finished = 1
            
        else:
            self.srcadd = ippkt_.src
            self.dstadd = ippkt_.dst
            self.srcport = ippkt_.data.sport
            self.dstport = ippkt_.data.dport
            self._finished = 0

        # update of flow ID
        self._curl2pkt = None
        self._curtime = 0
        self._dir = "fclient"
        self._idx = flow.idx
        self._isorphan = True
        self._skip = False
        flow.idx = flow.idx + 1
        self.marked = {}
        self.attributes = {}
        self.info = {}
        
    def isorphan(self):
        return self._isorphan

    def setorphan(self, o):
        self._isorphan = o

    def skip(self):
        return self._skip

    def setskip(self, s):
        self._skip = s
        
    def setcurl2pkt(self, pkt):
	self._curl2pkt = pkt
        
    def getcurl2pkt(self):
	return str( self._curl2pkt )

    def setcurtime(self, time):
        self._curtime = time

    def getcurtime(self):
        return self._curtime

    def gettcpheaderlen(self):
        pkt = self._curl2pkt
        ipl = ( dpkt.ethernet.Ethernet( pkt ).ip.v_hl & 0x0f ) * 4
        tcpl = ( dpkt.ethernet.Ethernet( pkt ).ip.tcp.off_x2 >> 4 ) * 4
        return (ipl, tcpl)

    def gettcppayloadlen(self):
        pkt = self._curl2pkt
        ip = dpkt.ethernet.Ethernet( pkt ).data
        tcpdata = ip.len-(len(ip.pack_hdr()))-ip.data.off_x2/4
        return tcpdata

    def getpktl3len(self):
        pkt = self._curl2pkt
        return dpkt.ethernet.Ethernet( pkt ).data.len

    def gettcpflags(self):
        pkt = self._curl2pkt
        return dpkt.ethernet.Ethernet( pkt ).ip.tcp.flags
    
    def flow2str(self):
        ipsrc_ = dnet.ip_ntoa(self.srcadd)
        ipdst_ = dnet.ip_ntoa(self.dstadd)
        ports_ = '%d %d' % (self.srcport, self.dstport)
        str = ipsrc_ + ' ' + ipdst_ + ' ' + ports_
        return str

    def flow2ascii(self):
        ipsrc_ = dnet.ip_ntoa(self.srcadd)
        ipdst_ = dnet.ip_ntoa(self.dstadd)
        return (ipsrc_, ipdst_, '%d' % self.srcport, '%d' % self.dstport)

    def ascii2flow(self, addrtuple):
        self.srcport = int(addrtuple[2])
	self.dstport = int(addrtuple[3])
	self.srcadd = dnet.ip_aton(addrtuple[0])
	self.dstadd = dnet.ip_aton(addrtuple[1])
    
    def setdir(self, dir_):
        self._dir=dir_
        
    def getdir(self):
        return self._dir
    
    def setfin(self, dir_):
        self._fin = dir_
        
    def getfin(self):
        return self._fin
    
    def isfclient(self):
        return self._dir == "fclient"
    
    def isfserver(self):
        return self._dir == "fserver"
    
    def isfinished(self):
        return self._finished

    def setfinished(self):
        self._finished = 1
        
    def haspacket(self):
        if self._curl2pkt == None:
            return False
        return True

