#!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# ABOUT

"""
 *
 *           i p c l a s s . p y
 *
 * ipclass.py follows Origin/Destination flows inside a packets stream coming from a pcap device
 *
 * Author -
 *
 *      Francesco Gringoli
 *      francesco.gringoli@ing.unibs.it
 *      University of Brescia
 *      1/3/2008
 *
 * Status -
 *      Stable
 * 
"""

#-------------------------------------------------------------------------------
# This file handles
#
# - the main program body

#-------------------------------------------------------------------------------
# TODO
#
# - clean the code
# - capture CTRL+'c'
# - seek fxml output to add header
# - define shutdown functions (modules) and use it

#-------------------------------------------------------------------------------
# import section

import sys
import dnet
import dpkt
import flow
import options
import modules
import util 

#-------------------------------------------------------------------------------
if __name__ == '__main__':

    # parse options
    opts = options.options()
    opts.parse(sys.argv)
    tlast = 0
    ctime = 0
    tcheck = 300 # seconds!
    timeout = 300 # seconds!
    
    # loop through the input module
    for flow in opts.inputmodule:

        # get current time
        ctime = flow.getcurtime()
        
        # check if we need to do garbage collection
        # flush input engine, use try, it can not support this
        # this step is not really mandatory but it is design clean!
        #try:
        #    opts.inputmodule.flushflow()
        #except:
        #    pass

        if ctime - tlast > tcheck:

            # go on only if this is not the first time we do
            if tlast != 0:
                
                if util.getverbosity() >= 1:
                    util.log( "\n-------------" )
                    util.log( "Starting garbage collection at " + str(ctime) )

                # loop over the db: use try as not all input modules support this
                try:
                    for gflow in opts.inputmodule.getdb():

                        flowtime = gflow.getcurtime()

                        if ctime - flowtime > timeout:

                            gflow.setfinished()

                            for mod in opts.outputmodules:
                                mod.dooutput( gflow )

                            # to prevent an erroneous re-insertion of the flow mark it as skip
                            gflow.setskip(True)
         
                except:
                    pass
                
                if util.getverbosity() >= 1:
                    util.log( "Terminating garbage collection:" )
                    util.log( "Flow in DB1 - " + str(opts.inputmodule.countflows()))
                    util.log( "Flow in DB2 - " + str(opts.inputmodule.realcountflows()))
                    util.log( "-------------\n" )

            tlast = ctime

        # process this flow
        for mod in opts.processmodules:
            mod.doprocess( flow )

        for mod in opts.outputmodules:
            mod.dooutput( flow )

            
    # we could still have flow into the db
    # use try as not all input modules could support this
    try:
        for flow in opts.inputmodule.getdb():
            flow.setfinished()

            for mod in opts.outputmodules:
                mod.dooutput( flow )
    except:
        pass

    # close open files
    opts.inputmodule.inputClose()

    for mod in opts.outputmodules:
        mod.outputClose()
