#!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#  ABOUT

"""
 *
 *           m o d a d d r e s s . p y
 *
 * This module handles TCP/IP addressing data of each flow
 *
 * Author -
 *
 *      Francesco Gringoli
 *      francesco.gringoli@ing.unibs.it
 *      University of Brescia
 *      1/3/2008
 *
 * Status -
 *      Stable
 *
 * Extended by -
 *
 *      Manuel Crotti
 *      manuel.crotti@ing.unibs.it
 *      University of Brescia
 *      which implemented the anonymization part
 *
"""

#-------------------------------------------------------------------------------
# This is the session creation module

#-------------------------------------------------------------------------------
# TODO
#
# - COMMENT THE CODE!!!
# - Check if data is compliant with the code
# - migliorare passaggio del tempo da pcap

#-------------------------------------------------------------------------------
# import section

import sys
import flow
import options
import modules
import dpkt
import math
import re
import socket

secret='\xa1\x02\xc3\xd4'

#-------------------------------------------------------------------------------
# This class defines the session processing module
#
# Method 'modinfo' returns the message help for this module
# Method 'default' returns the dictionary with default config for this module
# Method __init__ initializes this module
# Method 'setup' passes the supplied configuration to this module
# Method 'doprocess' performs processing task for this module given a flow
#
class modaddress(modules.template):
    def modinfo( self ):
        info = ( "address module, call with -m address:[anon=0|1]\n" +
                 "  where 'anon=1' setup the module to anonymize the tcp/ip addresses\n" )
        return info

    def default( self ):
        return {"anon":"0"}
    
    def __init__( self ):
        # call the base class setup function
        modules.template.setup( self )

        # initialize parameters
        self.processedflows = 0
        self.anon = 0
        
    def setup( self, config ):

        # if anonymization requested:
        if config["anon"] == "1":
            self.anon = 1
            
        # append processing capability inside the list
        self.capabilities.append( 'P' )

    def flowgetdata( self, flow ):

        return self.flowattr( flow )
        
    def attrdefault( self ):

        attrib=dict()
        attrib["anon"] = 0

        data=dict()

        return (attrib, data)
    
    def doprocess( self, flow ):

        (modAttrib, modData) = self.flowattr( flow )
        
        if flow.haspacket():
            
            # if we have never seen this flow modData is empty: fill it

            if "sip" not in modData:
                
                (sip, dip, sport, dport ) = flow.flow2ascii()
                
                if self.anon:
                    anonsip=""
                    anondip=""
                    asip=socket.inet_aton(sip)
                    adip=socket.inet_aton(dip)
                    for i in range (len(secret)):
                        anonsip+=str(eval(hex(ord(asip[i])))^eval(hex(ord(secret[i]))))+"."
                        anondip+=str(eval(hex(ord(adip[i])))^eval(hex(ord(secret[i]))))+"."
                    sip=anonsip[:-1]
                    dip=anondip[:-1]
                    modAttrib["anon"] = 1
                
                modData["sip"] = sip
                modData["dip"] = dip
                modData["sport"] = sport
                modData["dport"] = dport

        else:
            # do something only if anon requested and flow still not anonymized
            if self.anon and modAttrb["anon"] == 0:
                anonsip=""
                anondip=""
                asip=socket.inet_aton( modData["sip"] )
                adip=socket.inet_aton( modData["dip"] )
                for i in range (len(secret)):
                    anonsip+=str(eval(hex(ord(asip[i])))^eval(hex(ord(secret[i]))))+"."
                    anondip+=str(eval(hex(ord(adip[i])))^eval(hex(ord(secret[i]))))+"."
                sip=anonsip[:-1]
                dip=anondip[:-1]
                modAttrib["anon"] = 1
                modData["sip"] = sip
                modData["dip"] = dip

