#!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#  ABOUT
#
#  download new signatures at http://l7-filter.sourceforge.net/protocols

"""
 *
 *           m o d l 7 s u b s e t . p y
 *
 * This module handle the pattern matching filtering of TCP flows
 * limiting the signatures to those that match the name of the application
 * that generated the flow and provided by modgtruth.py
 *
 * Author -
 *
 *      Maurizio Dusi
 *      maurizio.dusi@ing.unibs.it
 *      University of Brescia
 *      January 2009
 *
 * Extended by -
 *
 *      Niccolo' Cascarano
 *      niccolo.cascarano@polito.it
 *      Politecnico di Torino
 *      January 2009
 *
 *
 * Status -
 *      Stable
 * 
"""

#-------------------------------------------------------------------------------
# This module implements the L7 pattern recognition

#-------------------------------------------------------------------------------
# TODO
#
# - move the patterns to another file
# - extend for "attribute" management
# - command line pattern do not wrk!
#-------------------------------------------------------------------------------
# import section

import sys
import flow
import options
import modules
import re
import dpkt
import util
import string


SUBSET_PROTOS = {
    "AquamacsEmacs": "http",
    "Calculator": "http",
    "DashboardClient": "http",
    "GoogleSoftwareUp": "ssl",
    "GoogleUpdateChec": "http",
    "GrowlHelperApp": "http",
    "HelpViewer": "http",
    "IEXPLORE": "http;ssl",
    "MATLAB": "ssl",
    "Mail": "http;pop3;pop3s;ssl",
    "MicrosoftMessen": "msn;ssl",
    "OmniSoftwareUpda": "http",
    "PubSubAgent": "http",
    "QuickTimePlayer": "http",
    "Safari": "http;ssl",
    "Skype": "skype;http",
    "SoftwareUpdate": "http",
    "SoftwareUpdateCh": "http",
    "SyndicationAgent": "http",
    "Synergy": "http",
    "Transmission": "bittorrent;http",
    "avast.setup": "http",
    "cvpnd": "skype",
    "dashboardadvisor": "http",
    "evolution": "ssl;pop3;pop3s;http;imap;smtp",
    "firefox": "http;ssl",
    "firefox-bin": "http;ssl",
    "gksu": "http",
    "gnome-panel": "http",
    "gtk-gnash": "http",
    "gweather-applet-2": "http",
    "http": "http",
    "iStatmenusBack": "http",
    "iTunes": "http",
    "kdeinit": "http;ssl",
    "ld-2.7.so": "http",
    "ntpd": "ntp",
    "ocspd": "http",
    "python2.5": "http",
    "ssh": "ssh",
    "svn": "ssl",
    "thunderbird": "http;ssl;pop3;pop3s;imap;smtp",
    "vmware": "http",
    "emule": "edonkey",
    "amule": "edonkey",
    "uTorrent": "bittorrent;http",
    "svchost": "dns",
    "freshclam": "http;dns",
    "googleearth": "http",
    "System": "http;dns",
    "Adobe_Updater": "http",
    "skypePM": "skype",
    "cupsd": "http", 
    "Ktorrent": "bittorrent;http",
    "kopete": "msn;ssl",
    "telnet": "telnet",
    "wget": "http",
    "rdesktop": "rdp",
    "ntpdate": "ntp",
    "TortoiseProc": "ssl",
    "PPLive": "pplive"
}

# <pattern> section
PATTERNS = {
    # tcp
    #enhanced
    "tcp":{
        "ftp": "^220.*ftp|^220.*(\x0d\x0a)$",
        #enhanced
        "smtp": "^220[\x09-\x0d -~]* (e?smtp|simple mail)|^(HELO|EHLO) .*\x0d\x0a",
        #enhanced
        "http": "http/(0\.9|1\.0|1\.1) [1-5][0-9][0-9]|(connect|post|get|head|propfind|mkcol|delete|put|copy|move|lock|unlock) [\x09-\x0d -~]*( http/[01]\.[019])?",
        "pop3": "^(\+ok [\x09-\x0d -~]*(ready|hello|pop|starting)|-err [\x09-\x0d -~]*(invalid|unknown|unimplemented|unrecognized|command))",
        "imap": "^(\* ok|a[0-9]+ noop)",
        # port 6436/6438
        "gnutella": "^(gnd[\x01\x02]?.?.?\x01|gnutella connect/[012]\.[0-9]\x0d\x0a|get /uri-res/n2r\?urn:sha1:|get /.*user-agent: (gtk-gnutella|bearshare|mactella|gnucleus|gnotella|limewire|imesh)|get /.*content-type: application/x-gnutella-packets|giv [0-9]*:[0-9a-f]*/|queue [0-9a-f]* [1-9][0-9]?[0-9]?\.[1-9][0-9]?[0-9]?\.[1-9][0-9]?[0-9]?\.[1-9][0-9]?[0-9]?:[1-9][0-9]?[0-9]?[0-9]?|gnutella.*content-type: application/x-gnutella|...................?lime)",
        #     #port 2703
        "msn": "ver [0-9]+ msnp[1-9][0-9]? [\x09-\x0d -~]*cvr0\x0d\x0a$|usr 1 [!-~]+ [0-9. ]+\x0d\x0a$|ans 1 [!-~]+ [0-9. ]+\x0d\x0a$",
        #     #port 1755 
        "msnfile": "^(ver [ -~]*msnftp\x0d\x0aver msnftp\x0d\x0ausr|method msnmsgr:)",
        "bittorrent": "^(\x13bittorrent protocol|azver\x01$|get /scrape\?info_hash=)|d1:ad2:id20:|\x08'7P\)[RP]",
        "ssh": "^ssh-[12]\.[0-9]",
        "ssl": "^(.?.?\x16\x03.*\x16\x03|.?.?\x01\x03\x01?.*\x0b)",
        "edonkey": "^[\xc5\xd4\xe3-\xe5].?.?.?.?([\x01\x02\x05\x14\x15\x16\x18\x19\x1a\x1b\x1c\x20\x21\x32\x33\x34\x35\x36\x38\x40\x41\x42\x43\x46\x47\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f\x50\x51\x52\x53\x54\x55\x56\x57\x58\x60\x81\x82\x90\x91\x93\x96\x97\x98\x99\x9a\x9b\x9c\x9e\xa0\xa1\xa2\xa3\xa4]|\x59................?[ -~]|\x96....$)",
        "telnet": "^\xff[\xfb-\xfe].\xff[\xfb-\xfe].\xff[\xfb-\xfe]",
        "rdp": "rdpdr.*cliprdr.*rdpsnd",
        "skype": "^..\x02.............",
	"pplive": "\x01...\xd3.+\x0c.$",
        },
    # ,
    # udp
    "udp":{
        "dns": "^.?.?.?.?.?[\x01\x02]\x00.?\x00.?\x00\x00.*\x00[\x00-\x1c]\x00\x01",
        "skype": "^..\x02.............",
        "rtp": "^\x80......?.?.?.?.?.?.?.?.?.?.?.?.?\x80",
        "srtp": "rtsp/1.0 200 ok",
        "snmp": "^\x02\x01\x04.+([\xa0-\xa3]\x02[\x01-\x04].?.?.?.?\x02\x01.?\x02\x01.?\x30|\xa4\x06.+\x40\x04.?.?.?.?\x02\x01.?\x02\x01.?\x43)",
        "dhcp": "^[\x01\x02][\x01- ]\x06.*c\x82sc",
        "bittorrent": "^(\x13bittorrent protocol|azver\x01$|get /scrape\?info_hash=)|d1:ad2:id20:|\x08'7P\)[RP]",
        "ipp": "ipp://",
        "xmpp": "<stream:stream[\x09-\x0d ][ -~]*[\x09-\x0d ]xmlns=['.?]jabber",
        "ntp":  "^([\x13\x1b\x23\xd3\xdb\xe3]|[\x14\x1c$].......?.?.?.?.?.?.?.?.?[\xc6-\xff])",
	"pplive": "\x01...\xd3.+\x0c.$",
        }
    }

# </patterns>
	
# <ports association> section
PORTS = {
	21: "ftp",
	25: "smtp",
	80: "http",
	8080: "http",
	110: "pop3",
	143: "imap",
	6436: "gnutella",
	6437:"gnutella",
	6438:"gnutella",
	1863: "msn",
	443: "ssl",
	22: "ssh",
	6881: "bittorrent",
	6882: "bittorrent",
	6883: "bittorrent",
	6884: "bittorrent",
	6885: "bittorrent",
	6886: "bittorrent",
	6887: "bittorrent",
	6888: "bittorrent",
	6889: "bittorrent",
	4661: "edonkey",
	4662: "edonkey",
	3635: "edonkey",
	9573: "edonkey",
	53: "dns",
	161: "snmp",
	162: "snmp",
	631: "ipp",
	5222: "xmpp",
	5269: "xmpp",
	}
# </ports association>
	


def packetmatch( pat, payload):
    return pat.findall(payload)

#-------------------------------------------------------------------------------
# This class defines the L7 parsing object module
#
# Method doprocess: process the TCP payload searching for known protocol patterns

class modl7subset(modules.template):
    def modinfo( self ):
        info = ( "l7subset module, call with -m l7subset:[pat=pattern list]:[filter=\"expr\"]:[optim=yes|no]\n" +
                 "\n"
                 "  where\n" +
                 "\n" +
                 "    pattern list is a comma separated list of pattern names\n" +
                 "\n" +
                 "    'expr' is a valid filtering expression on the 'proto' variable.\n" +
                 "\n" +
                 "  This module will mark all flow that match with protocols as in \"pattern list\"\n" )
        return info

    def default( self ):
        return {"pat":None, "filter":None, "optim":"no"}
    
    def __init__( self ):
        
        # call the base class setup function
        modules.template.setup( self )

    def attrdefault( self ):

        # return default attribute for flow initialization
        attrib = dict()
        attrib["matchedprotocols"] = 0
        attrib["matchedprotocolsdir"] = {"fclient":0, "fserver":0}
        attrib["totpkts"]=0
        attrib["countfclient"]=0
        attrib["countfserver"]=0

        #attrib["proto"]="NoMatch"

        data=dict()
        
        return (attrib, data)

    def setup( self, config ):

        # get configuration
        #if config['pat'] != None:
        #    self.pat = re.split( '[\s,]+', config['pat'].strip() )
        #else:
        #self.pat = PATTERNS.keys()

        # append processing capability inside the list
        self.capabilities.append( 'P' )

        # check if filter given
        #if config['filter']:
        #
        #    self.filter = config['filter']
        #
        #    # loop on all proto names
        #    for pn in PATTERNS.keys():
        #        self.filter = self.filter.replace(pn,"\""+pn+"\"")
        #
        #    # now replace the proto keyword
        #    self.filter = self.filter.replace("proto","modAttrib[\"proto\"]")
        #    
        #else:
        self.filter = None
	    
        self.compiledPat = {"tcp":{}, "udp":{}}

	#protolist = self.pat

        for l4 in ["tcp", "udp"]:
            for pname in PATTERNS[l4].keys():
                self.compiledPat[l4][pname] = re.compile(PATTERNS[l4][pname], re.IGNORECASE)

        # check if we should optimize pattern matching based on port heuristic
        self.optim = config['optim']

        if self.optim != "yes" and self.optim != "no":
            util.err( "Error in optim parameter, check the code" )
            util.exit(-1)
    
    def flowgetdata( self, flow ):

        return self.flowattr( flow )
    
    def doprocess( self, flow ):

        (modAttrib, modData) = self.flowattr( flow )

        # Move what follows in the init or load method
        # -------------------------------------------
        try:
            flow.attributes["gtruth"]
        except:
            #util.err("modl7subset: load modgtruth first!\n")
            return None
        # -------------------------------------------


        if flow.haspacket():

            # get packet and l4 proto
            pkt = flow.getcurl2pkt()
            ip = dpkt.ethernet.Ethernet( pkt ).data
            l4proto = ip.p
            l4name = None
            if l4proto == 6:
                l4name = "tcp"
            else:
                l4name = "udp"
                
            # adjust global counters for this flow
            modAttrib["totpkts"] += 1
            isfclient = False
            if ( flow.getdir() == "fclient" ):
                modAttrib["countfclient"] += 1
                isfclient = True
            else:
                modAttrib["countfserver"] += 1
                isfclient=False
                
            # extract packet's payload
            tcp = ip.data
            payload = tcp.data
            # what happens with UDP? This is not well documented on dpkt

            # if we requested optimization based on port heuristic execude this block
            if self.optim == "yes":

                # We do not perform pattern match over a session already matched
                # (This should give a great speedup)
                if modAttrib["matchedprotocols"] > 0:
                  return

                # check for payload match for the preferred protocol (depends on the port)
                # only if we didn't match before
                if tcp.sport in PORTS:
                    if PORTS[ tcp.sport ] not in modData:
                        if packetmatch( self.compiledPat[l4name][ PORTS[ tcp.sport ] ], payload ):
                            # increase matching counter
                            modAttrib["matchedprotocols"] += 1

                            # ok matched, mark
                            modData[PORTS[ tcp.sport ]] = {}

                            if isfclient:
                                modData[PORTS[ tcp.sport ]]["dir"] = "fclient"
                                modData[PORTS[ tcp.sport ]]["numpkts"] = modAttrib["countfclient"]

                            else:
                                modData[PORTS[ tcp.sport ]]["dir"] = "fserver"
                                modData[PORTS[ tcp.sport ]]["numpkts"] = modAttrib["countfserver"]

                            return

                elif tcp.dport in PORTS:
                    if PORTS[ tcp.dport ] not in modData:
                        if packetmatch( self.compiledPat[l4name][ PORTS[ tcp.dport ] ], payload ):
                            # increase matching counter
                            modAttrib["matchedprotocols"] += 1

                            # ok matched, mark
                            modData[PORTS[ tcp.dport ]] = {}

                            if isfclient:
                                modData[PORTS[ tcp.dport ]]["dir"] = "fclient"
                                modData[PORTS[ tcp.dport ]]["numpkts"] = modAttrib["countfclient"]

                            else:
                                modData[PORTS[ tcp.dport ]]["dir"] = "fserver"
                                modData[PORTS[ tcp.dport ]]["numpkts"] = modAttrib["countfserver"]

                            return

            # ok we can get here if optimization was disabled (default) or because optimized matching didn't find anything

            # Ok, I will be honest, check for the first match along both directions and stop when found one
            if modAttrib["matchedprotocols"] > 0:
                return

            # first check if we had already a match for both directions
            if isfclient and modAttrib["matchedprotocolsdir"]["fclient"] > 0:
                return

            if not isfclient and modAttrib["matchedprotocolsdir"]["fserver"] > 0:
                return
            
            # ------------------------------------------------------------
            (gt_attrib, gt_data)  = flow.attributes["gtruth"]
            gt_appname = gt_attrib["appname"]
            protolist = []
            app_found = 0

            # Search the protocol subset specific for this appication name
            for subproto in SUBSET_PROTOS:
                m = re.search(subproto.lower(), gt_appname.replace(' ','').lower())
                if m:
                    app_found = 1
                    plist = SUBSET_PROTOS[subproto].split(";")

                    # we found a subset of procotols for this application
                    # walk the list with all the possible protocols for this application
                    # and if we have a signature for this protocol add it to the
                    # "protolist" list
                    for pname in plist:
                        if self.compiledPat[l4name].has_key(pname):
                            protolist.append(pname)
                    break

            # in this case we did not found any subset of protocols for this application
            # use as protolist the entire set of protocols' signatures and
            # switch on the "app_not_found" indicator
            if not app_found:
                protolist = PATTERNS[l4name].keys()
                modData["app_not_found"] = {}
 
            # in this case we found the application and a subset of protocols to test for it
            # but we do not have any valid signature for those protocols
            if app_found and (len(protolist) == 0):
                modData["proto_not_found"] = {}
            # ------------------------------------------------------------

            for pname in protolist:

                # test if match against pname ONLY if it didn't match against it yet
                if pname not in modData:

                    # get the pattern for this protocol
                    pat = self.compiledPat[l4name][pname]

                    if packetmatch( pat, payload ):

                        # increase matching counter
                        modAttrib["matchedprotocols"] += 1
                        
                        # ok matched, mark
                        modData[pname] = {}
                        
                        if isfclient:
                             modData[pname]["dir"] = "fclient"
                             modData[pname]["numpkts"] = modAttrib["countfclient"]
                             modAttrib["matchedprotocolsdir"]["fclient"] += 1
                             
                        else:
                             modData[pname]["dir"] = "fserver"
                             modData[pname]["numpkts"] = modAttrib["countfserver"]
                             modAttrib["matchedprotocolsdir"]["fserver"] += 1
 
