#!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#  ABOUT

"""
 *
 *           m o d t c p . p y
 *
 * This module handles TCP flows
 *
 * Author -
 *
 *      Francesco Gringoli
 *      francesco.gringoli@ing.unibs.it
 *      University of Brescia
 *      11/12/2008
 *
 * Status -
 *      Stable
 *
"""

#-------------------------------------------------------------------------------
# This is the session creation module

#-------------------------------------------------------------------------------
# TODO
#
# - COMMENT THE CODE!!!

#-------------------------------------------------------------------------------
# import section

import sys
import flow
import options
import modules
import dpkt
import math
import re
import socket
import util

#-------------------------------------------------------------------------------
# This class defines the session processing module
#
# Method 'modinfo' returns the message help for this module
# Method 'default' returns the dictionary with default config for this module
# Method __init__ initializes this module
# Method 'setup' passes the supplied configuration to this module
# Method 'doprocess' performs processing task for this module given a flow
#
class modtcp(modules.template):
    def modinfo( self ):
        info = ( "tcp module, call with -m tcp\n" +
                 "  \n" )
        return info

    def default( self ):
        return {}
    
    def __init__( self ):
        # call the base class setup function
        modules.template.setup( self )
        
    def setup( self, config ):

        # append processing capability inside the list
        self.capabilities.append( 'P' )

    def flowgetdata( self, flow ):

        (modAttrib, modData) = self.flowattr( flow )
        status = modAttrib["status"]
        return ({"l4":"tcp"}, {"status":status})
        
    def attrdefault( self ):

        attrib = {"istcp":0, "status":0}
        data   = {"finaddr":None}

        return (attrib, data)
    
    def doprocess( self, flow ):

        (modAttrib, modData) = self.flowattr( flow )
        
        if flow.haspacket():

            # first check this is TCP: this should be more efficient... do this test before!
            if modAttrib["istcp"] == 1 or dpkt.ethernet.Ethernet( flow.getcurl2pkt() ).data.p == 6:

                # do this each time...
                modAttrib["istcp"] = 1

                # extract address
                ippkt = dpkt.ethernet.Ethernet( flow.getcurl2pkt() ).data
                ( dstadd, srcadd ) = ( ippkt.dst, ippkt.src )

                # extract the tcp part
                tcp = dpkt.ethernet.Ethernet( flow.getcurl2pkt() ).data.data
                
                flags = tcp.flags
                SYN = flags & 2
                ACK = flags & 16
                FIN = flags & 1
                RST = flags & 4

                # extract status
                status = modAttrib["status"]
                
                # if only SYN, TCP flow can only be in status 0?
                if SYN and not ACK:
                    # we know this flow and we see another SYN in the same direction, start again
                    # otherwise err

                    if flow.isfclient():
                        if status != 0:
                            if util.getverbosity() >= 2:
                                util.log( flow.flow2str() + " New SYN from client on a connected flow: reset flow" )
                        else:
                            if util.getverbosity() >= 2:
                                util.log( flow.flow2str() + " New SYN again: reset flow" )
                        
                    else:
                        util.err( "New SYN from server: wrong direction, fix it!" )
                        util.exit(-1)

                    if util.getverbosity() >= 2:
                        util.log( flow.flow2str() + " New SYN: set flow status to 0" )
                    status = 0

                # 3-way-handshake ends. Check SYN+ACK direction, if wrong exception
                elif SYN and ACK:
                    if flow.isfclient():
                        if util.getverbosity() >= 2:
                            util.err( flow.flow2str() + " SYN+ACK: wrong direction, skipping" )
                        flow.setskip(True)
                    
                    elif status == 0:
                        if util.getverbosity() >= 2:
                            util.log( flow.flow2str() + " Connected: set flow status to 1" )
                        status  = 1
                    else:
                        if util.getverbosity() >= 2:
                            util.log( flow.flow2str() + " SYN+ACK on an already connected flow: don't care" )

                # we are still in status 0: setting skip will prevent successive processing
                # modules to analyze this flow and will force no insertion of this flow
                # into the database
                elif status == 0:
                    if util.getverbosity() >= 2:
                        util.log( flow.flow2str() + " Received a packet before 3whs-end: skip flow" )
                    flow.setskip(True)

                # RST handling, setting finished flag will cause output module to produce output
                # and the flow will be removed automatically at next packet extraction
                elif RST:
                    if util.getverbosity() >= 2:
		        util.log( flow.flow2str() + " RST End" )
                    flow.setfinished()

                # we have FIN in status 1: wait a FIN from the peer
                elif FIN and status == 1:
                    if util.getverbosity() >= 2:
                        util.log( flow.flow2str() + " First FIN: set flow status to 2" )
                    status = 2
                    modAttrib["finaddr"] = srcadd

                # we have the FIN from the peer: now waits for an ACK
                elif FIN and status == 2 and modAttrib["finaddr"] == dstadd:
                    if util.getverbosity() >= 2:
                        util.log( flow.flow2str() + " Second and opposite FIN: set flow status to 3" )
                    status = 3
                    modAttrib["finaddr"] = srcadd

                # this ACK follows the second FIN: if in status 3 shutdown the connection
                elif ACK and status == 3 and modAttrib["finaddr"] == dstadd:
                    if util.getverbosity() >= 2:
                        util.log( flow.flow2str() + " FIN End" )
                    flow.setfinished()

                modAttrib["status"] = status

