#!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#  ABOUT

"""
 *
 *           m o d u l e s . p y
 *
 * modules.py contains the code to handle module loading
 *
 * Author -
 *
 *      Francesco Gringoli
 *      francesco.gringoli@ing.unibs.it
 *      University of Brescia
 *      1/3/2008
 *
 * Status -
 *      Stable
 * 
"""

import sys
import flow
import util

#-------------------------------------------------------------------------------
# TODO
#
# - clean up access to flow attributes and marked (define functions in flow class?)

#-------------------------------------------------------------------------------
# This class defines basic methods common to all modules
#
# capabilities can take values
#  - I:   this modules can be used to collect data to process
#  - O:   this modules can be passed flows to produce information
#  - P:   this modules can be supplied with flows to analyze and mark

class template( object ):
    def modinfo( self ):
	return "template module, do not use"

    def __init__( self, cmdline = None ):
        self.cmdline = cmdline
        self.modname = 'template'
        self.setup()

    def setup( self ):
        self.verbosity = 0
        self.cycles = 0
	self.capabilities = []
        self.errormsg = None
        self.verbose = 0

    def name( self ):
        return self.modname

    def setname( self, name ):
        self.modname = name

    def isonerror( self ):
        if self.errormsg:
            return True

    def errorstr( self ):
        return self.errormsg

    def isinput( self ):
        if 'I' in self.capabilities:
            return True
        return False
    
    def isoutput( self ):
        if 'O' in self.capabilities:
            return True
        return False

    def isprocess( self ):
        if 'P' in self.capabilities:
            return True
        return False

    def doprocess( self, flow ):
        self.cycles = self.cycles + 1

    def dooutput( self, flow ):
        self.cycles = self.cycles + 1
        
    def postprocess( self ):
        pass

    def shutdown( self ):
        pass

    def sign( self ):
        return 'ipclass'

    def attrdefault( self ):
        return None

    def flowattr( self, flow ):
        if self.modname not in flow.attributes:
            flow.attributes[self.modname] = self.attrdefault()
        flow.marked[self.modname] = self
        return flow.attributes[self.modname]

    def flowgetdata( self, flow ):
        return flow.attributes[self.modname]

    def flowsetdata( self, flow, modname, data ):
        # add a template module to collect back
        # data if the target module has not been
        # instantiated
        tmod = template()
        tmod.setname( modname )

        # insert data
	flow.attributes[modname] = data

        # append this to the mark list
        flow.marked[modname] = tmod

    def flowmarked( self, flow ):
        return flow.marked

    def outputClose( self ):
        pass

    def inputClose( self ):
        pass

#-------------------------------------------------------------------------------
# This method allocates a requested ipclass module after verifying that
# it is a module. The module is configurated with the passed configuration
#

def load( modname ):
    _modname = "mod" + modname
    newmod = None

    if util.getverbosity() > 0:
        exec('import ' + _modname )
        newmod = eval( _modname + '.' + _modname + '()' )
    else:
        try:
            exec('import ' + _modname )
            newmod = eval( _modname + '.' + _modname + '()' )
        except:
            util.err( "Error loading module " + modname )
	    util.err( "Try to increase verbosity to detect the actual error" )
            return None

    if newmod.sign() == 'ipclass':
        return newmod
    else:
        return None
