#!/usr/bin/env python

# COPYRIGHT NOTICE

"""
* Copyright (c) 2008, University of Brescia, Brescia, IT
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Brescia nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF BRESCIA ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL UNIVERSITY OF BRESCIA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#  ABOUT

"""
 *
 *           m o d u s e r o u t . p y
 *
 * This module can be freely modify by users to implement their output function
 *
 * Author -
 *
 *      Francesco Gringoli
 *      francesco.gringoli@ing.unibs.it
 *      University of Brescia
 *      14/9/2008
 *
 * Acknowledges to Maurizio Dusi and Niccolo Cascarano for their debugging help!      
 *
 *
 * Status -
 *      stable
 *
"""

#-------------------------------------------------------------------------------
# This module handles I/O concerning user-defined file format

#-------------------------------------------------------------------------------
# TODO
#

#-------------------------------------------------------------------------------
# import section

import sys
import flow
import options
import modules
import string
import math
import util
from os import path

#-------------------------------------------------------------------------------
# This class implements a prototype for users
#
# Method 'modinfo' returns the message help for this module
# Method 'default' returns the dictionary with default config for this module
# Method __init__ initializes this module
# Method 'setup' passes the supplied configuration to this module
# Method 'dooutput' performs output task for this module given a flow

class moduserout(modules.template):
    def modinfo( self ):
        info = ( self.modname + " module, call with -m " + self.modname +
                 " [of=output_file_name]\n" +
                 " [l7sub=yes|no]")
        return info

    def default( self ):
        return {"of":"", "l7sub":"no"}
    
    def __init__( self ):
        # call the base class setup function
        modules.template.setup( self )

        self.writtenFlows = 0
        
    def __iter__( self ):
        return self

    def setup( self, config ):

        # if requested allocates basic structures for output and set 'O' output capability
        self.output = config['of']
        self.ofile = open( self.output, 'w' )
        self.capabilities.append( 'O' )

        # should we perform l7sub output?
        self.l7sub = config['l7sub']

        # check if valid
        if self.l7sub != "yes" and self.l7sub != "no":
            util.err( "Error in l7sub parameter, adjust the code please" )
            util.exit(-1)
        
    def dooutput( self, flow ):

        if flow.isfinished():
            # should we skip this flow?
            if flow.skip():
                return

            # we need addresses, times (first packet) and l7 token
            mods = self.flowmarked( flow )

            #(sessA, sessD) = mods["session"].flowgetdata( flow )
            (addrA, addrD) = mods["address"].flowgetdata( flow )
            (l7tkA, l7tkD) = mods["l7parser"].flowgetdata( flow )
            (gtruthA, gtruthD) = mods["gtruth"].flowgetdata( flow )

            if self.l7sub == "yes":
                (l7subtkA, l7subtkD) = mods["l7subset"].flowgetdata( flow )

            protostr = ""
            for proto in l7tkD:
                protostr = protostr + proto + ";"

            if len(protostr) > 0:
                protostr = protostr.rstrip()

            else:
                protostr = "NOMATCH"

            protosubstr = ""

            if self.l7sub == "yes":
                for proto in l7subtkD:
                    protosubstr = protosubstr + proto + ";"

                if len(protosubstr) > 0:
                    protosubstr = protosubstr.rstrip()

                else:
                    protosubstr = "NOMATCH"
            
            #t0 = sessD["fclient"]["time0"]
            src = addrD["sip"]
            dst = addrD["dip"]
            spt = addrD["sport"]
            dpt = addrD["dport"]
            app = gtruthA["appname"]
            dur = str(float(gtruthA["time"]) - float(gtruthA["time0"]))
            dur2 = str(float(gtruthA["lasttime"]) - float(gtruthA["time0"]))
            sby = str(gtruthA["fserver"])
            cby = str(gtruthA["fclient"])
            l4p = gtruthA["l4proto"]

            # convert time to hex notation
            t0 = (gtruthA["time0"])
            ssec = int ( math.floor( t0 ) )
            usec = int ( round ( (t0 - ssec) * 1000000.0 ) )
            time0= '%08X.%08X' % (ssec, usec)

            if self.l7sub == "yes":
                self.ofile.write( time0 + ":" + src + ":" + dst + ":" + spt + ":" + dpt + ":" + protostr + ":" + app + ":" + protosubstr )
            else:
                self.ofile.write( time0 + ":" + src + ":" + dst + ":" + spt + ":" + dpt + ":" + protostr + ":" + app )
            self.ofile.write( ":" + dur + ":" + dur2 + ":" + sby + ":" + cby + ":" + l4p + "\n" )

    def outputClose( self ):

        self.ofile.close()
                        
